.XCREF
.XLIST

PushRegs	MACRO	RegList
	IRP	reg,<RegList>
	push	reg
	ENDM
		ENDM

PopRegs		MACRO	RegList
	IRP	reg,<RegList>
	pop	reg
	ENDM
		ENDM

SetZero		MACRO Reg
	xor	Reg,Reg		;Reg = 0
		ENDM

SetPtr		MACRO	P, S, O
	mov	P.Ofst,	O	;set offset
	mov	P.Segm,	S	;set segment
		ENDM

SetPtrByOfst	MACRO	P, S, O
	mov	ax,Offset O
	mov	P.Ofst,	ax	;set offset
	mov	P.Segm,	S	;set segment
		ENDM

GetPtr		MACRO	GPtr
	les	DI, GPtr	;Load ES:DI with GPtr
		ENDM

GetDSPtr	MACRO	GDPtr
	lds	SI, GDPtr	;Load DS:SI with GDPtr
		ENDM

NullJump	MACRO
	jmp SHORT $+2		;jump to next instruction
		ENDM

GetVector	MACRO	VecNum,	PtrVar
	mov	ah,35h		;Get vector
	mov	al,VecNum	;al = vector number
	int	21h		;call DOS
	SetPtr	PtrVar,	ES, BX	;save the vector in PtrVar
		ENDM

JmpFar		MACRO JAddr
	jmp	dword ptr JAddr	;Jump far to JAddr
		ENDM

CallFar		MACRO CAddr
	call	dword ptr CAddr	;Call far to CAddr
		ENDM

;Work around bug in the	POPF instruction on certain 80286 chips
FakePOPF	MACRO
	LOCAL X1, X2
	jmp	SHORT X2	;skip over IRET
X1:	iret			;pop flags and do a RETF
X2:	push	cs		;fake a	FAR CALL
	call	X1
		ENDM

WaitForRetrace	MACRO
	LOCAL WaitNoH, WaitH, Go
	;Note DX must be initialized to	03DAh
	cli			;Interrupts off
WaitNoH:
	in	al,dx		;Get 6845 status
	test	al,8		;Check for vertical retrace
	jnz	Go		;In progress? go
	shr	al,1		;Wait for end of horizontal
	jc	WaitNoH		; retrace
WaitH:
	in	al,DX		;Get 6845 status again
	shr	al,1		;Wait for horizontal
	jnc	WaitH		; retrace
Go:
		ENDM


WordMoveNoSnow	MACRO
	LOCAL Wait1, Wait2, Go,	Next
	;Note DX must be initialized to	03DAh, CX to loop count
	shl	cx,1		;Words to bytes
Next:
	cli			;Interrupts off
Wait1:
	in	al,dx		;Get 6845 status
	test	al,8		;Check for vertical retrace
	jnz	Go		;In progress? go
	shr	al,1		;Wait for end of horizontal
	jc	Wait1		; retrace
Wait2:
	in	al,dx		;Get 6845 status again
	shr	al,1		;Wait for horizontal
	jnc	Wait2		;  retrace
Go:	movsb			;Move one byte
	sti			;Allow interrupts
	loop	Next		;Move next byte
		ENDM


FastMoveNoSnow	MACRO
	LOCAL Wait1, Wait2, Go,	Next
	;Note DX must be initialized to	03DAh, CX to loop count
Next:
	cli			;Interrupts off
Wait1:
	in	al,dx	;Get 6845 status
	test al,8		;Check for vertical retrace
	jnz	Go		;In progress? go
	shr	al,1		;Wait for end of horizontal
	jc	Wait1	; retrace
Wait2:
	in	al,dx	;Get 6845 status again
	shr	al,1		;Wait for horizontal
	jnc	Wait2		;  retrace
Go:	movsw			;Move one word
	sti			;Allow interrupts
	loop	Next		;Move next byte
		ENDM

VideoPrim	MACRO
	push	bp		;save BP
	int	10h		;call BIOS
	pop	bp		;restore BP
		ENDM

VideoCall	MACRO	VidService
	mov	ah, VidService	;service number
	VideoPrim		;primitive form
		ENDM

VidCallAX	MACRO	VidServAX
	mov	ax, VidServAX	;service number
	VideoPrim		;primitive form
		ENDM

KbdCall		MACRO	KbdService
	mov	ah, KbdService	;service number
	int	16h		;BIOS keyboard interrupt
		ENDM

DosCall		MACRO	DosFuncNum
	mov	ah,DosFuncNum	;ah = Function number
	int	21h		;Call DOS
		ENDM

DosCallAX	MACRO	AXval
	mov	ax,AXval	;ah = Function #, al has modifier
	int	21h		;Call DOS
		ENDM

WordToCGA	MACRO	MVreg
	mov	ax,MVreg	;Move video word into ax
	stosw		; and then to screen
	sti			;Allow interrupts
		ENDM

;saves all basic registers in order of a Registers variable,
;but assumes that flags	are already pushed
SaveAllNoFlags	MACRO
	push	es	
	push	ds
	push	di
	push	si
	push	bp
	push	dx
	push	cx
	push	bx
	push	ax
		ENDM

;Save all registers, including flags
SaveAllRegs	MACRO
	pushf			;save flags
	SaveAllNoFlags		;save all basic	registers
		ENDM

;Restore all registers saved in	order of SaveAllRegs
RestoreAllRegs	MACRO
	pop	ax
	pop	bx
	pop	cx
	pop	dx
	pop	bp
	pop	si
	pop	di
	pop	ds
	pop	es
	FakePOPF
		ENDM

;Reset the PIC with an end of interrupt	command	sent to	the proper port.
ResetPIC	MACRO
	mov	al,20h		;end of	interrupt to PIC
	out	20h,al		;send it
		ENDM

;Reset the keyboard on an IBM PC
ResetKbd	MACRO
	in	al,61h		;get keyboard control
	mov	ah,al		;in ah and al
	or	al,80h		;reset keyboard
	out	61h,al		;back out
	NullJump		;delay
	mov	al,ah		;original value
	out	61h,al		;keyboard is reset
		ENDM

;******************************************************	Equates

True		=	1
False	=	0
WP		EQU	WORD PTR

;constants for referring to window coordinates

XLow		=	(BYTE PTR 0)
YLow		=	(BYTE PTR 1)
XHigh	=	(BYTE PTR 0)
YHigh	=	(BYTE PTR 1)

;display types

UnknownD	=	-1
Unenhanced	=	0
Mono		=	0
CGA		=	1
MCGA		=	2
EGA		=	3
VGA		=	4
PGC		=	5

;miscellaneous constants and equates

Font8x8		=	256
TurboBlink	=	16
Blink		=	10000000b

;******************************************************	Structures

;Structure of a	pointer
Pointer	STRUC
	Ofst	DW	0
	Segm	DW	0
Pointer	ENDS

.CREF
.LIST
